=head1 NAME

Gdraw2::CairoTrackGroup - track group impelemented using Cairo

=head1 SYNOPSIS
 
 use Gdraw2::CairoTrackGroup;
 
 my $group = Gdraw2::CairoTrackGroup->new(
     start => $group_start,
     end   => $group_end
 );
 
 # add to a page
 $page->addChild($group);

=head1 DESCRIPTION

The concrete group class implemented using Cairo 2D drawing library. See Gdraw2::AbstractTrackGroup for more information.

=cut

package Gdraw2::CairoTrackGroup;

use Moose;
use Gdraw2::AbstractTrackGroup;
use Gdraw2::Util::SharedObject;


use Carp qw/cluck/;

extends 'Gdraw2::AbstractTrackGroup';

override draw => sub {
	my $self = shift;
	my $cr = $self->parent->cairoContext;
	
	$self->arrangeAll;
	
	if ($self->isCircular) {
		$cr->save;
		foreach my $track ($self->getChildren) {
			$track->draw;
		}
		$cr->restore;
	}
	else {
		# debug
		my $page = $self->parent;
		
		# group 
#		$cr->save;
#		$cr->set_source_rgba(0,1,0,0.5);
#		$cr->set_line_width(1);
#		$cr->move_to(0,0);
#		$cr->line_to($page->width - $self->leftMargin - $self->rightMargin, 0);
#		$cr->stroke;
#		$cr->restore;
		##########################################
		
		# we have been translated to group init site
		my $track_count = 0;
		foreach my $track ($self->getChildren) {
			$cr->save;
			$cr->translate(0,$track->trackCascade);
			$track->draw;
			
			# debug
#			if ($track_count>0) {
#				my $gstart = $track->graphicStart;
#				my $gend   = $track->graphicEnd;
#				my $name_c = $track->calcNameCascade($track->numCascadeNameLines);
#				$cr->set_source_rgba(0.5,1,0.5,0.5);
#				$cr->rectangle($gstart,-$name_c-$self->trackDistance,$gend-$gstart,$self->trackDistance);
#				$cr->fill;
#			}
			###############################
			
			$cr->restore;
		} continue {$track_count++}
	}
};
1;

=head1 SEE ALSO

 Gdraw2::AbstractTrackGroup

=cut
