package Gdraw2::Element::AbstractRuler;

use threads::shared;
use common::sense;
use Moose;
use Gdraw2::Element::AbstractElement;

use POSIX qw/floor/;

extends 'Gdraw2::Element::AbstractElement';
with 'Gdraw2::Element::RoleArrangeable';

has '+showName' => ( default => 0 );

has tickSize => (
	is        => 'rw',
	metaclass => 'Recordable',
	isa       => 'Num',
	default   => 2
);

has gridLength => (
	is        => 'rw',
	metaclass => 'Recordable',
	isa       => 'Int',
	default   => 1000
);

has integralTick => (
	is        => 'rw',
	metaclass => 'Recordable',
	isa       => 'Bool',
	default   => 1
);

has abbrevTickText => (
	is        => 'rw',
	metaclass => 'Recordable',
	isa       => 'Bool',
	default   => 1
);

sub tickPositionList {
	my $self = shift;
	my @result;

	my $start = $self->start;
	my $end   = $self->end;
	my $step  = $self->gridLength;

	push @result, $start;

	my $cyc_start;
	if ( $self->integralTick ) {
		$cyc_start = ( floor( $start / $step ) + 1 ) * $step;
	}
	else {
		$cyc_start = $start + $step;
	}

	for ( my $i = $cyc_start ; $i < $end ; $i += $step ) {
		if ( $i == 0 ) {
			push @result, 1;
		}
		else {
			push @result, $i;
		}
	}

	push @result, $end;
	return @result;
}

sub abbrev {
	my $self = shift;
	my $num  = shift;

	my $str;
	if ( $num >= 1e9 and $num % 1e9 == 0 ) {
		$str = int( $num / 1e9 ) . 'G';
		return $str;
	}
	elsif ( $num >= 1e6 and $num % 1e6 == 0 ) {
		$str = int( $num / 1e6 ) . 'M';
		return $str;
	}
	elsif ( $num >= 1000 and $num % 1000 == 0 ) {
		$str = int( $num / 1e3 ) . 'K';
		return $str;
	}
	else {
		return $num;
	}
}

1;
