package Gdraw2::Element::CairoDataCurve;

use Moose;
use Gdraw2::Util qw/polar_point/;
use List::Util qw/max min/;

extends 'Gdraw2::Element::AbstractDataCurve';

with 'Gdraw2::Element::RoleCairoElement';
with 'Gdraw2::Element::RoleCairoDefaultNameLead';

#override drawNameLeadLine => sub { };

override draw => sub {
    my $self = shift;

    my $gstart = $self->graphicStart;
    my $gend   = $self->graphicEnd;
    my @data   = @{ $self->data };
    my $data_l = $self->singleDataGraphicLength;

    my $track = $self->parent;
    my $group = $track->parent;
    my $cr    = $self->getTrackPage->cairoContext;

    my $w = $track->elementWidth;

    my $min = min @data,$self->middleLine;
    my $max = max @data,$self->middleLine;

    my $mid_height = ( $self->middleLine - $min ) / ( $max - $min ) * $w;

    if ( $self->isCircular ) {
        my $r = $self->radius;

        my $circ_shape = sub {
            $cr->new_path;
            $cr->move_to( polar_point( $gstart, $r - $w / 2 + $mid_height ) );

            my $theta = $gstart;
            for ( my $i = 0 ; $i < @data ; $i++ ) {
                my $curr_height = ( $data[$i] - $min ) / ( $max - $min ) * $w;
                $cr->arc(0,0,$r-$w/2+$curr_height,$theta,$theta+$data_l);
                $theta += $data_l;
            }
            $cr->line_to( polar_point( $gend, $r - $w / 2 + $mid_height ) );
            $cr->arc_negative( 0, 0, $r - $w / 2 + $mid_height, $gend,
                $gstart );
            $cr->close_path;
        };

        # clip inner half
        $cr->save;
        {
            $cr->new_path;
            $cr->arc( 0, 0, $r - $w / 2, $gstart, $gend );
            $cr->arc_negative( 0, 0, $r-$w/2+$mid_height, $gend, $gstart );
            $cr->close_path;
            $cr->clip;

            &$circ_shape;
            $self->setContextColor( $cr, $self->fillColor );
            $cr->fill;
        }
        $cr->restore;

        # clip outer half
        $cr->save;
        {
            $cr->new_path;
            $cr->arc( 0, 0, $r-$w/2+$mid_height, $gstart, $gend );
            $cr->arc_negative( 0, 0, $r + $w / 2, $gend, $gstart );
            $cr->close_path;
            $cr->clip;

            &$circ_shape;
            $self->setContextColor( $cr, $self->fillColor2 );
            $cr->fill;
        }
        $cr->restore;
    }
    else {
        my $linear_shape = sub {
            my $x = $gstart;
            $cr->move_to( $gstart, $w / 2 - $mid_height );
            foreach my $value (@data) {
                my $curr_h = ( $value - $min ) / ( $max - $min ) * $w;
                $cr->line_to( $x, $w / 2 - $curr_h );
                $x += $data_l;
                $cr->line_to( $x, $w / 2 - $curr_h );
            }
            $cr->line_to( $gend, $w / 2 - $mid_height );
            $cr->close_path;
        };

        # clip upper half
        $cr->save;
        {
            $cr->move_to( $gstart, $w / 2 - $mid_height  );
            $cr->line_to( $gstart, -$w / 2 );
            $cr->line_to( $gend,   -$w / 2 );
            $cr->line_to( $gend,   $w / 2 - $mid_height );
            $cr->close_path;
            $cr->clip;

            &$linear_shape();
            $self->setContextColor( $cr, $self->fillColor );
            $cr->fill;
        }
        $cr->restore;

        # clip lower half
        $cr->save();
        {
            $cr->move_to( $gstart, $w / 2 - $mid_height );
            $cr->line_to( $gstart, $w / 2 );
            $cr->line_to( $gend,   $w / 2 );
            $cr->line_to( $gend,   $w / 2 - $mid_height );
            $cr->close_path;
            $cr->clip;

            &$linear_shape();
            $self->setContextColor( $cr, $self->fillColor2 );
            $cr->fill;
        }
        $cr->restore;

    }
};

1;
