package Gdraw2::Element::CairoMapping;

use threads::shared;
use common::sense;
use Moose;
extends 'Gdraw2::Element::AbstractMapping';
with 'Gdraw2::Element::RoleCairoElement';
with 'Gdraw2::Element::RoleCairoDefaultNameLead';

use Gdraw2::Util qw/polar_point/;

override draw => sub {
	my $self = shift;

	my $cr = $self->getTrackPage->cairoContext;

	my $track = $self->parent;
	my $group = $track->parent;
	my $page  = $group->parent;

	my $ext_group = $page->getChildAt( $self->extGroupIndex );
	my $ext_track = $ext_group->getChildAt( $self->extTrackIndex );

	my $halfw     = $track->elementWidth / 2;
	my $ext_halfw = $ext_track->elementWidth / 2;

	if ( $page->isCircular ) {
		my $r     = $track->radius;
		my $ext_r = $ext_track->radius;

		my $gstart = $self->graphicStart;
		my $gend   = $self->graphicEnd;

		my $ext_gstart =
		  $ext_group->calcGraphicPosition( $self->extStart ) -
		  $ext_group->bitGraphicLength;
		my $ext_gend = $ext_group->calcGraphicPosition( $self->extEnd );

		my ( $r1, $r2 );    # outer radius
		my ( $r3, $r4 );    # inner radius
		my ( $g1, $g2 );    # outer graphical position
		my ( $g3, $g4 );    # inner graphical position

		#-- determine actual drawing points --#

		# current track at outside
		if ( $r > $ext_r ) {
			$r1 = $r + $halfw;
			$r2 = $r - $halfw;
			$r3 = $ext_r + $ext_halfw;
			$r4 = $ext_r - $ext_halfw;
			$g1 = $gstart;
			$g2 = $gend;
			$g3 = $ext_gstart;
			$g4 = $ext_gend;
		}

		# current track at inside
		else {
			$r1 = $ext_r + $ext_halfw;
			$r2 = $ext_r - $ext_halfw;
			$r3 = $r + $halfw;
			$r4 = $r - $halfw;
			$g1 = $ext_gstart;
			$g2 = $ext_gend;
			$g3 = $gstart;
			$g4 = $gend;
		}

		#-- fill --#
		$cr->save;

		$cr->move_to( polar_point( $g1, $r2 ) );
		$cr->arc( 0, 0, $r1, $g1, $g2 );
		$cr->line_to( polar_point( $g2, $r2 ) );
		$cr->line_to( polar_point( $g4, $r3 ) );
		$cr->arc_negative( 0, 0, $r4, $g4, $g3 );
		$cr->line_to( polar_point( $g3, $r3 ) );
		$cr->close_path;

		my $fill = $self->fillColor;
		if ( $fill->isa('Gdraw2::Util::RGBAColor') ) {
			$cr->set_source_rgba( $fill->flat );
		}
		else {
			$cr->set_source_rgb( $fill->flat );
		}
		$cr->fill;

		$cr->restore;

		#-- stroke --#
		$cr->save;

		$cr->move_to( polar_point( $g3, $r4 ) );
		$cr->line_to( polar_point( $g3, $r3 ) );
		$cr->line_to( polar_point( $g1, $r2 ) );
		$cr->line_to( polar_point( $g1, $r1 ) );

		$cr->move_to( polar_point( $g4, $r4 ) );
		$cr->line_to( polar_point( $g4, $r3 ) );
		$cr->line_to( polar_point( $g2, $r2 ) );
		$cr->line_to( polar_point( $g2, $r1 ) );

		$cr->set_line_width( $self->lineWidth );

		my $stroke = $self->strokeColor;
		if ( $stroke->isa('Gdraw2::Util::RGBAColor') ) {
			$cr->set_source_rgba( $stroke->flat );
		}
		else {
			$cr->set_source_rgb( $stroke->flat );
		}
		$cr->stroke;

		$cr->restore;
	}
	else {

		# first of all, we need to get the location of external track
		my $ext_left = $ext_group->leftMargin - $group->leftMargin;
		my $ext_top =
		  ( $ext_group->groupCascade + $ext_track->trackCascade ) -
		  ( $group->groupCascade + $track->trackCascade );

		my $ext_g_start =
		  $ext_left +
		  $ext_group->calcGraphicPosition( $self->extStart ) -
		  $ext_group->bitGraphicLength;
		my $ext_g_end =
		  $ext_left + $ext_group->calcGraphicPosition( $self->extEnd );

		# determine actual drawing points
		my ( $x1, $x2, $x3, $x4, $y1, $y2, $y3, $y4 );

		if ( $ext_top < 0 ) {
			$x1 = $ext_g_start;
			$x2 = $ext_g_end;
			$x3 = $self->graphicStart;
			$x4 = $self->graphicEnd;
			$y1 = $ext_top - $ext_halfw;
			$y2 = $ext_top + $ext_halfw;
			$y3 = -$halfw;
			$y4 = $halfw;
		}
		else {
			$x1 = $self->graphicStart;
			$x2 = $self->graphicEnd;
			$x3 = $ext_g_start;
			$x4 = $ext_g_end;
			$y1 = -$halfw;
			$y2 = $halfw;
			$y3 = $ext_top - $ext_halfw;
			$y4 = $ext_top + $ext_halfw;
		}

		#-- fill --#

		$cr->save;

		$cr->move_to( $x1, $y2 );
		$cr->line_to( $x1, $y1 );

		$cr->line_to( $x2, $y1 );
		$cr->line_to( $x2, $y2 );

		$cr->line_to( $x4, $y3 );
		$cr->line_to( $x4, $y4 );

		$cr->line_to( $x3, $y4 );
		$cr->line_to( $x3, $y3 );

		$cr->close_path;

		my $fill = $self->fillColor;
		if ( $fill->isa('Gdraw2::Util::RGBAColor') ) {
			$cr->set_source_rgba( $fill->flat );
		}
		else {
			$cr->set_source_rgb( $fill->flat );
		}
		$cr->fill;

		$cr->restore;

		#-- stroke --#
		$cr->save;

		$cr->move_to( $x1, $y1 );
		$cr->line_to( $x1, $y2 );
		$cr->line_to( $x3, $y3 );
		$cr->line_to( $x3, $y4 );

		$cr->move_to( $x2, $y1 );
		$cr->line_to( $x2, $y2 );
		$cr->line_to( $x4, $y3 );
		$cr->line_to( $x4, $y4 );

		$cr->set_line_width( $self->lineWidth );

		my $stroke = $self->strokeColor;
		if ( $stroke->isa('Gdraw2::Util::RGBAColor') ) {
			$cr->set_source_rgba( $stroke->flat );
		}
		else {
			$cr->set_source_rgb( $stroke->flat );
		}
		$cr->stroke;

		$cr->restore;
	}
};

1;
