package Gdraw2::Element::CairoPair;

use Moose;
use common::sense;
extends 'Gdraw2::Element::AbstractPair';
with 'Gdraw2::Element::RoleCairoElement';

use Gdraw2::Util qw/polar_point/;

override draw => sub {
	my $self = shift;

	my $track = $self->parent;
	my $group = $track->parent;
	my $cr    = $self->getTrackPage->cairoContext;

	my $bitSize = $group->bitGraphicLength;

	my @stroke_rgb = (
		$self->strokeColor->red, $self->strokeColor->green,
		$self->strokeColor->blue
	);

	$cr->save;
	$cr->set_line_width( $self->lineWidth );
	$cr->set_line_join('round');
	$cr->set_source_rgb(@stroke_rgb);

	if ( $self->isCircular ) {
		my $r0 = $self->radius - $track->elementWidth / 2;
		my $r1 =
		  $track->radius +
		  $track->calcCascade( $track->numCascadeLines - 1 ) +
		  $track->elementWidth / 2;

		foreach my $curr ( @{ $self->position } ) {
			my $angle;
			if ( $curr > 0 ) {
				$angle = $group->calcGraphicPosition($curr) - $bitSize / 2;
			}
			else {
				$angle = $group->calcGraphicPosition($curr) + $bitSize / 2;
			}
			$cr->move_to( polar_point( $angle, $r0 ) );
			$cr->line_to( polar_point( $angle, $r1 ) );
		}
	}
	else {
		confess "linear not implemented";
	}

	$cr->stroke;
	$cr->restore;
};

override drawNameLeadLine => sub {
	my $self = shift;

	my $track = $self->parent;
	my $group = $track->parent;
	my $page  = $group->parent;
	my $cr    = $page->cairoContext;

	my $bitSize = $group->bitGraphicLength;

	if ( $self->isCircular ) {
		if ( $self->showName and length( $self->displayName ) > 0 ) {
			my $r_anchor = $self->anchorRadius;

			my $angle     = $self->anchorGraphicPosition;
			my $angle_cos = cos $angle;

			my $text_cr_f = $page->textCairoFont;
			my $ext       = $text_cr_f->text_extents( $self->displayName );

			my $max_radius = $page->maxRadius;
			my $vertex_x   = $max_radius;
			$vertex_x *= -1 if $angle_cos <= 0;

			my $name_x =
			    $angle_cos > 0
			  ? $vertex_x + $page->groupDistance
			  : $vertex_x - $page->groupDistance;
			my $name_y = $self->circularNameCascade;

			$cr->save;

			for my $pos ( @{ $self->position } ) {
				my $gpos;
				if ( $pos > 0 ) {
					$gpos = $group->calcGraphicPosition($pos) - $bitSize / 2;
				}
				else {
					$gpos = $group->calcGraphicPosition($pos) + $bitSize / 2;
				}
				$cr->move_to( polar_point( $gpos, $r_anchor ) );
				$cr->line_to( $vertex_x, $name_y );
			}

			$cr->move_to( $vertex_x, $name_y );
			$cr->line_to( $name_x, $name_y );

			$cr->set_line_width( 0.6 * $self->lineWidth );
			$cr->stroke;
			$cr->restore;
		}
	}
	else {
		if ( $self->showName and length( $self->displayName ) > 0 ) {
			confess "needs to be implemented";
		}
	}
};

1;
