package Gdraw2::Element::CairoRuler;

use Moose;
use common::sense;
use Math::Trig qw/pi pi2 pi4 pip2 pip4/;
use Gdraw2::Util qw/polar_point/;

extends 'Gdraw2::Element::AbstractRuler';
with 'Gdraw2::Element::RoleCairoElement';
with 'Gdraw2::Element::RoleCairoDefaultNameLead';

override draw => sub {
	my $self = shift;

	my $tick = $self->tickSize;

	my $group   = $self->getTrackGroup;
	my $page    = $group->parent;
	my $cr      = $page->cairoContext;
	my $cr_font = $page->textCairoFont;

	my @list_pos   = sort { $a <=> $b } $self->tickPositionList;
	my @list_g_pos = map  { $group->calcGraphicPosition( $_ - 0.5 ) } @list_pos;
	my @list_tick_text;
	if ( $self->abbrevTickText ) {
		@list_tick_text = map { $self->abbrev($_) } @list_pos;
	}
	else {
		@list_tick_text = @list_pos;
	}

	$cr->save;

	# set stroke color
	my $stroke = $self->strokeColor;
	if ( $stroke->isa('Gdraw2::Util::RGBAColor') ) {
		$cr->set_source_rgba( $stroke->red, $stroke->green, $stroke->blue,
			$stroke->alpha );
	}
	else {
		$cr->set_source_rgb( $stroke->red, $stroke->green, $stroke->blue );
	}

	# set stroke line width
	$cr->set_line_width( $self->lineWidth );
	$cr->set_line_join('round');

	# make the drawing
	if ( $page->isCircular ) {
		my $r = $self->radius;

		# ruler main line
		$cr->save;
		$cr->arc( 0, 0, $r, $list_g_pos[0], $list_g_pos[-1] );
		$cr->stroke;
		$cr->restore;

		# ticks
		for ( my $i = 0 ; $i < @list_pos ; $i++ ) {
			my $gpos = $list_g_pos[$i];

			# tick line
			$cr->save;
			$cr->move_to( polar_point( $list_g_pos[$i], $r ) );
			$cr->line_to( polar_point( $list_g_pos[$i], $r + $tick ) );
			$cr->stroke;
			$cr->restore;

			# tick text
			my $ext = $cr_font->text_extents( $list_tick_text[$i] );

			$cr->save;
			my ( $pt_x, $pt_y ) = polar_point( $gpos, $r + $tick );

			# lower half
			if ( sin($gpos) > 0 ) {
				if ( cos($gpos) > 0 ) {
					$cr->move_to( $pt_x, $pt_y + $ext->{height} );
				}
				else {
					$cr->move_to( $pt_x - $ext->{width},
						$pt_y + $ext->{height} );
				}
			}

			# upper half
			else {
				if ( cos($gpos) > 0 ) {
					$cr->move_to( $pt_x, $pt_y );
				}
				else {
					$cr->move_to( $pt_x - $ext->{width}, $pt_y );
				}
			}
			$cr->show_text( $list_tick_text[$i] );

			$cr->restore;
		}
	}
	else {
		my $cascade = $self->cascade;

		# ruler main line
		$cr->save;
		$cr->move_to( $list_g_pos[0], $cascade );
		$cr->line_to( $list_g_pos[-1], $cascade );
		$cr->stroke;
		$cr->restore;

		# ticks
		for ( my $i = 0 ; $i < @list_pos ; $i++ ) {

			# tick line
			$cr->save;
			$cr->move_to( $list_g_pos[$i], $cascade );
			$cr->line_to( $list_g_pos[$i], $cascade - $tick );
			$cr->stroke;
			$cr->restore;

			# tick text
			my $extent = $cr_font->text_extents( $list_tick_text[$i] );
			my $x =
			  $list_g_pos[$i] - $extent->{x_bearing} - $extent->{width} / 2;
			my $y = $cascade - $tick * 2;

			$cr->save;
			$cr->move_to( $x, $y );
			$cr->show_text( $list_tick_text[$i] );
			$cr->restore;
		}
	}

	$cr->restore;
};

1;
