package Gdraw2::Element::RoleCairoDefaultNameLead;

use common::sense;
use threads::shared;
use Moose::Role;

use Carp qw/cluck/;
use Gdraw2::Util qw/polar_point/;

around drawNameLeadLine => sub {
	my $orig = shift;
	my $self = shift;
	
	my $track = $self->parent;
	my $page  = $self->getTrackPage;
	my $cr    = $page->cairoContext;
	
	my $track_gstart = $track->graphicStart;
	my $track_gend   = $track->graphicEnd;
	
	if ( $self->isCircular ) {

		if ( $self->showName and length( $self->displayName ) > 0 ) {
		    #say "draw name lead for ",$self->displayName;
			my $angle     = $self->anchorGraphicPosition;
			my $angle_cos = cos $angle;
			
			my $text_cr_f = $page->textCairoFont;
			my $ext = $text_cr_f->text_extents( $self->displayName );
			
			my ( $anchor_x, $anchor_y ) = polar_point($angle,$self->anchorRadius);

			my $max_radius = $page->maxRadius;
			my $vertex_x   = $max_radius;
			$vertex_x *= -1 if $angle_cos <= 0;

			my $name_x =
			    $angle_cos > 0
			  ? $vertex_x + $page->groupDistance
			  : $vertex_x - $page->groupDistance;
			my $name_y    = $self->circularNameCascade;

			$cr->save;
			$cr->set_line_width( 0.6 * $self->lineWidth );
			$self->setContextColor( $cr, $self->strokeColor );
			#say "\t$anchor_x, $anchor_y";
			#say "\t$vertex_x, $name_y";
			#say "\t$name_x,   $name_y";
			$cr->move_to( $anchor_x, $anchor_y );
			$cr->line_to( $vertex_x, $name_y );
			$cr->line_to( $name_x,   $name_y );
			$cr->stroke;
			$cr->restore;
		}
	}
	else {
		if ( $self->showName and length( $self->displayName ) > 0 ) {
			my $y = $self->nameCascade;
			my $anchor  = $self->anchorGraphicPosition;
			goto DRAW_END if $anchor<$track_gstart or $anchor>$track_gend;
			
			my $cascade = 0;
			if ( $self->meta->does_role('Gdraw2::Element::RoleArrangeable') ) {
				$cascade = $self->cascade;
			}
			
			$cr->save;
			$cr->set_line_width( 0.6 * $self->lineWidth );
			$self->setContextColor( $cr, $self->strokeColor );
			$cr->move_to( $anchor, $cascade );
			$cr->line_to( $anchor, -$y );
			$cr->stroke;
			$cr->restore;
			DRAW_END:
		}
	}
	
};

1;
