package Gdraw2::Element::RoleCairoElement;

use common::sense;
use threads::shared;
use Moose::Role;

use Carp qw/cluck/;
use Gdraw2::Util qw/polar_point/;
use Gdraw2::Element::AbstractElement;

around drawName => sub {
	my $orig  = shift;
	my $self  = shift;
	my $track = $self->parent;
	my $page  = $self->getTrackPage;
	my $cr    = $page->cairoContext;
	my $text_cr_f = $page->textCairoFont;
	
	my $track_gstart = $track->graphicStart;
	my $track_gend   = $track->graphicEnd;

	# test if we are arranged
	confess "Element <" . $self->displayName . "> not arranged before drawing"
	  if !$self->arrangeOK;

	if ( $self->isCircular ) {
		if ( $self->showName and length( $self->displayName ) > 0 ) {
			my $angle     = $self->anchorGraphicPosition;
			my $angle_cos = cos $angle;

			my $ext = $text_cr_f->text_extents( $self->displayName );

			my $max_radius = $page->maxRadius;

			my $name_x =
			    $angle_cos > 0
			  ? $max_radius + $page->groupDistance
			  : -$max_radius - $page->groupDistance;
			my $name_y = $self->circularNameCascade;

			$cr->save;
			$cr->set_scaled_font($text_cr_f);
			if ( $angle_cos > 0 ) {
				$cr->move_to( $name_x, $name_y + $ext->{height} / 2 );
			}
			else {
				$cr->move_to( $name_x - $ext->{width},
					$name_y + $ext->{height} / 2 );
			}

			$self->textColor->applyToCairoContext($cr);
			$cr->show_text( $self->displayName );
			$cr->restore;
		}
	}
	else {

		# assume we have been translated to track start position:
		# (left_margin, track_dev)

		# draw name
		if ( $self->showName and length( $self->displayName ) > 0 ) {
			my ($nameStart,$nameEnd) = $self->nameGraphicPosition;
			
			goto NAME_END if $nameEnd<$track_gstart or $track_gend<$nameStart; 
			
			my $y = $self->nameCascade;

			my $anchor  = $self->anchorGraphicPosition;
			my $cascade = 0;
			if ( $self->meta->does_role('Gdraw2::Element::RoleArrangeable') ) {
				$cascade = $self->cascade;
			}

			$cr->save;
			$cr->set_scaled_font($text_cr_f);
			$cr->move_to( $nameStart, -$y );
			$self->textColor->applyToCairoContext($cr);
			$cr->show_text( $self->displayName );
			$cr->restore;
			
			NAME_END:
		}
	}
};

around nameGraphicLength => sub {
	my $orig = shift;
	my $self = shift;
	my $extents =
	  $self->getTrackPage->textCairoFont->text_extents( $self->displayName );
	return $extents->{width};
};

around nameGraphicPosition => sub {
	my $orig       = shift;
	my $self       = shift;
	my $name_width = $self->nameGraphicLength;

	if ( $self->isCircular ) {
		confess "circular graphics should not use this method";
	}

	my $ele_anchor_gpos = $self->anchorGraphicPosition;

	my $start;
	my $end;

	given ( $self->nameAnchor ) {
		when ('left') {
			$start = $ele_anchor_gpos;
			$end   = $ele_anchor_gpos + $name_width;
		}
		when ('center') {
			$start = $ele_anchor_gpos - $name_width / 2;
			$end   = $ele_anchor_gpos + $name_width / 2;
		}
		when ('right') {
			$start = $ele_anchor_gpos - $name_width;
			$end   = $ele_anchor_gpos;
		}
	}
	return ( $start, $end );
};

sub setContextColor {
	my $self = shift;
	my $cr = shift;
	my $color = shift;
	if ( $color->isa('Gdraw2::Util::RGBAColor') ) {
		$cr->set_source_rgba( $color->flat );
	}
	else {
		$cr->set_source_rgb( $color->flat );
	}
}

1;

