package Gdraw2::Util::RoleAbstractParent;

use threads::shared;
use Moose::Role;
use Gdraw2::Util::RoleAbstractChild;
use Carp qw/cluck/;

has children => (
	is      => 'bare',
	isa     => 'ArrayRef',
	default => sub { return shared_clone [] }
);

sub addChild {
	my $self  = shift;
	my $child = shift;
	$self->addChildAt( $child, $self->numChildren );
}

sub addChildAt {
	my $self  = shift;
	my $child = shift;
	my $index = shift;

	confess "$child not a RoleAbstractChild"
	  if !$child->meta->does_role('Gdraw2::Util::RoleAbstractChild');

	if ( $self->hasChild($child) ) {
		cluck "attempt to add exist child: ", $child->addr;
		return;
	}

	# length check
	my $max = $self->numChildren;
	if ( $index > $max ) {
		cluck "index $index out of children num $max, append child to tail";
		push @{ $self->{children} }, $child;
	}
	elsif ($index==$max) {
		push @{ $self->{children} }, $child;
	}
	else {

		# TODO: thread sharing support
		if ($threads::threads) {
			my $new = shared_clone [];

			my $i = 0;
			foreach my $curr ( @{ $self->{children} } ) {
				push @$new, $child if $i == $index;
				push @$new, $curr;
			}
			continue { $i++ }

			$self->{children} = $new;
		}
		else {
			splice @{ $self->{children} }, $index, 0, $child;
		}
	}
}

sub hasChild {
	my $self  = shift;
	my $child = shift;
	foreach my $curr ( @{ $self->{children} } ) {
		return 1
		  if ( $curr->equals($child) );
	}
	return 0;
}

sub childIndex {
	my $self  = shift;
	my $child = shift;

	my $num = $self->numChildren;
	for ( my $i = 0 ; $i < $num ; $i++ ) {
		my $curr = $self->{children}[$i];
		if ($curr->equals($child)) {
			return $i;
		}
	}
	return -1;
}

sub removeChild {
	my $self  = shift;
	my $child = shift;
	my $max   = $self->numChildren;
	for ( my $i = 0 ; $i < $max ; $i++ ) {
		if ( $child->equals( $self->{children}[$i] ) ) {
			return $self->removeChildAt($i);
		}
	}
	cluck "attempt to remove non-exist child: ", $child->addr;
}

sub removeChildAt {
	my $self  = shift;
	my $index = shift;

	# length check
	my $max = $self->numChildren;
	if ( $index >= $max ) {
		cluck "index $index out of range: $max";
		return;
	}

	# thread support: shared array cannot be spliced
	if ($threads::threads) {
		my $new = shared_clone [];
		my $deleted;
		for ( my $i = 0 ; $i < $max ; $i++ ) {
			if ( $i eq $index ) {
				$deleted = $self->{children}[$i];
			}
			else {
				push @$new, $self->{children}[$i];
			}
		}
		$self->{children} = $new;
		return $deleted;
	}
	else {
		return splice @{ $self->{children} }, $index, 1;
	}
}

sub removeAllChildren {
	my $self = shift;
	my @tmp  = @{ $self->{children} };
	$self->{children} = shared_clone [];
	return @tmp;
}

sub getChildAt {
	my $self  = shift;
	my $index = shift;
	my $max   = $self->numChildren;
	if ( $index >= $max ) {
		cluck "index $index out of range: $max";
		return;
	}
	return $self->{children}[$index];
}

sub getChildren {
	return @{ $_[0]->{children} };
}

sub numChildren {
	return scalar @{ $_[0]->{children} };
}

1;
