package Moose::Meta::Attribute::Custom::Recordable;
use Gdraw2::Meta::Attribute::Recordable;
sub register_implementation { 'Gdraw2::Meta::Attribute::Recordable' }

package Gdraw2::Util::RoleRecordable;

use common::sense;
use mro 'c3';
use threads::shared;

use Moose::Role;

use Gdraw2::Util qw/load_module/;

use Scalar::Util qw/blessed/;

use YAML::Syck;

sub loadHash {
    my $self  = shift;
    my $store = shift;
    my %extra = @_;

    # get class
    confess "No class information on store" if !exists $store->{CLASS};
    my $class = $store->{CLASS};

    # prepare attributes
    my %attr_ready;
    foreach my $key ( keys %$store ) {
        next if $key eq 'CLASS';
        my $value = $store->{$key};
        if ( my $value_type = ref $value ) {
            if ( $value_type eq 'HASH' and exists $value->{CLASS} ) {
                my $value_class = $value->{CLASS};
                load_module($value_class);
                $attr_ready{$key} = $value_class->loadHash($value);
            }
            else {
                $attr_ready{$key} = Load Dump $store->{$key};
            }
        }
        else {
            $attr_ready{$key} = $store->{$key};
        }
    }
    $attr_ready{$_} = $extra{$_} foreach keys %extra;

    # create object
    my $obj = $class->new(%attr_ready);

    return $obj;
}

sub toHash {
    my $self = shift;

    my $store = shared_clone {};
    $store->{CLASS} = blessed $self;

    my $meta  = $self->meta;
    my $class = blessed $self;

    my @linear_isa = @{ mro::get_linear_isa( $class, 'c3' ) };
    my @super_meta = map { $_->meta } @linear_isa;

    foreach my $curr_meta ( $meta, @super_meta ) {
        foreach my $attr_name ( $curr_meta->get_attribute_list ) {
            my $attr = $curr_meta->get_attribute($attr_name);

            if ( !$attr->isa('Gdraw2::Meta::Attribute::Recordable') ) {
                next;
            }

            my $value = $self->{$attr_name};
            if ( ref $value ) {
                if ( blessed $value
                    and $value->meta->does_role('Gdraw2::Util::RoleRecordable')
                  )
                {
                    $store->{$attr_name} = $self->{$attr_name}->toHash;
                }
                else {
                    $store->{$attr_name} = Load Dump $self->{$attr_name};
                }
            }
            else {
                $store->{$attr_name} = $self->{$attr_name};
            }
        }
    }

    return $store;
}

sub getParams {
    my $hash = toHash(@_);
    return $hash;
}

sub applyParams {
    my ( $self, $params ) = @_;

    my $meta  = $self->meta;
    my $class = blessed $self;

    my @linear_isa = @{ mro::get_linear_isa( $class, 'c3' ) };
    my @super_meta = map { $_->meta } @linear_isa;

    foreach my $curr_meta ( $meta, @super_meta ) {
        foreach my $attr_name ( $curr_meta->get_attribute_list ) {
            next if !exists $params->{$attr_name};

            my $attr = $curr_meta->get_attribute($attr_name);
            next if !$attr->isa('Gdraw2::Meta::Attribute::Recordable');

            my $value = $self->{$attr_name};
            if ( ref $value ) {
                if ( blessed $value
                    and $value->meta->does_role('Gdraw2::Util::RoleRecordable')
                  )
                {
                    $self->{$attr_name}->applyParams( $params->{$attr_name} );
                }
                else {
                    $self->{$attr_name} = Load Dump $params->{$attr_name};
                }
            }
            else {
                $self->{$attr_name} = $params->{$attr_name};
            }
        }
    }
}

1;
