package Gdraw2Gtk::Act;

use Moose;

has task => (
    is       => 'ro',
    isa      => 'Gdraw2Gtk::Task',
    weak_ref => 1,
    required => 1
);

sub redo {
    confess "override this function";
}

sub undo {
    confess "override this function";
}

package Gdraw2Gtk::Act::Indexed;
use Moose;
extends 'Gdraw2Gtk::Act';

has indices => (
    is       => 'rw',
    isa      => 'ArrayRef[Int]',
    required => 1
);

package Gdraw2Gtk::Act::Move;

use Moose;
use Moose::Util::TypeConstraints;

extends 'Gdraw2Gtk::Act::Indexed';

has direction => (
    is       => 'rw',
    isa      => enum( [ 'up', 'down' ] ),
    required => 1
);

override redo => sub {
    my $self    = shift;
    my @indices = @{ $self->indices };
    $self->task->move_at( @indices, $self->direction );
};

override undo => sub {
    my $self = shift;

    my @indices_u = @{ $self->indices };
    my $direction_u;

    if ( $self->direction eq 'up' ) {
        $direction_u = 'down';
        $indices_u[-1]--;
    }
    else {
        $direction_u = 'up';
        $indices_u[-1]++;
    }

    $self->task->move_at( @indices_u, $direction_u );
};

package Gdraw2Gtk::Act::Config;
use Moose;

extends 'Gdraw2Gtk::Act::Indexed';

has 'params_old' => (
    is       => 'rw',
    isa      => 'HashRef',
    required => 1
);

has 'params_new' => (
    is       => 'rw',
    isa      => 'HashRef',
    required => 1
);

sub _apply {
    my ( $self, $params ) = @_;

    my @indices = @{ $self->indices };

    my $obj = $self->task->get_object_at(@indices);
    $obj->applyParams($params);
}

override redo => sub {
    my $self = shift;
    $self->_apply( $self->params_new );
};

override undo => sub {
    my $self = shift;
    $self->_apply( $self->params_old );
};

package Gdraw2Gtk::Act::Add;
use Moose;
use Moose::Util::TypeConstraints;

extends 'Gdraw2Gtk::Act::Indexed';

has 'object' => (
    is       => 'rw',
    required => 1
);

override redo => sub {
    my $self    = shift;
    my @indices = @{ $self->indices };
    my $obj     = $self->object;

    if ( @indices == 3 ) {
        $self->task->add_element_at( $obj, @indices );
    }
    elsif ( @indices == 2 ) {
        $self->task->add_track_at( $obj, @indices );
    }
    elsif ( @indices == 1 ) {
        $self->task->add_group_at( $obj, @indices );
    }
    else {
        confess "invalid indices: @indices";
    }
};

override undo => sub {
    my $self = shift;

    my @indices = @{ $self->indices };

    if ( @indices == 3 ) {
        $self->task->remove_element_at(@indices);
    }
    elsif ( @indices == 2 ) {
        $self->task->remove_track_at(@indices);
    }
    elsif ( @indices == 1 ) {
        $self->task->remove_group_at(@indices);
    }
    else {
        confess "invalid indices: @indices";
    }
};

package Gdraw2Gtk::Act::Remove;
use Moose;
use Moose::Util::TypeConstraints;

extends 'Gdraw2Gtk::Act::Indexed';

has 'object' => (
    is       => 'rw',
    required => 1
);

override redo => sub {
    my $self = shift;

    my $task    = $self->task;
    my @indices = @{ $self->indices };

    if ( @indices == 3 ) {
        $task->remove_element_at(@indices);
    }
    elsif ( @indices == 2 ) {
        $task->remove_track_at(@indices);
    }
    elsif ( @indices == 1 ) {
        $task->remove_group_at(@indices);
    }
    else {
        confess "invalid indices: @indices";
    }
};

override undo => sub {
    my $self = shift;

    my $obj     = $self->object;
    my $task    = $self->task;
    my @indices = @{ $self->indices };
    
    if ( @indices == 3 ) {
        $self->task->add_element_at( $obj, @indices );
    }
    elsif ( @indices == 2 ) {
        $self->task->add_track_at( $obj, @indices );
    }
    elsif ( @indices == 1 ) {
        $self->task->add_group_at( $obj, @indices );
    }
    else {
        confess "invalid indices: @indices";
    }
};

1;
