package Gdraw2Gtk::BlastImportConfig;

use strict;
use feature qw/say switch/;

use File::ShareDir qw/:ALL/;
use Glib qw/TRUE FALSE/;
use Gtk2;

use Moose;
use Gdraw2::Element::CairoBlock


has uiBuilder => (
    is      => 'ro',
    isa     => 'Gtk2::Builder',
    default => sub { return Gtk2::Builder->new }
);

sub BUILD {
    my $self = shift;

    # load GUI resources
    my $builder = $self->uiBuilder;
    my $file_ui = dist_file( 'Gdraw2', 'BlastImporter.glade' );
    $builder->add_from_file($file_ui);

    # connect signals
    $builder->connect_signals( undef, $self );

    # update UI
    $self->on_BlastImporter_EValueCheck_toggled;
    $self->on_BlastImporter_BitScoreCheck_toggled;
}

sub run {
    my $self = shift;

    my $builder = $self->uiBuilder;
    my $dialog  = $builder->get_object('BlastImporter');

    my $re = $dialog->run;
    $dialog->hide;

    return $re;
}

sub doFiltEValue {
    my $self    = shift;
    my $builder = $self->uiBuilder;
    return $builder->get_object('BlastImporter_EValueCheck')->get_active;
}

sub doFiltBitScore {
    my $self    = shift;
    my $builder = $self->uiBuilder;
    return $builder->get_object('BlastImporter_BitScoreCheck')->get_active;
}

sub eValueCutoff {
    my $self    = shift;
    my $builder = $self->uiBuilder;
    return $builder->get_object('BlastImporter_EValueEntry')->get_text;
}

sub bitScoreCutoff {
    my $self    = shift;
    my $builder = $self->uiBuilder;
    return $builder->get_object('BlastImporter_BitScoreSpin')->get_value;
}

sub stackHSPs {
    my $self    = shift;
    my $builder = $self->uiBuilder;

    if ( $builder->get_object('BlastImporter_TrackLayout')->get_active ) {
        return 0;
    }
    elsif ( $builder->get_object('BlastImporter_ElemLayout')->get_active ) {
        return 1;
    }
    else {
        confess "";
    }
}

sub on_BlastImporter_EValueCheck_toggled {
    my $self    = shift;
    my $builder = $self->uiBuilder;

    my $check  = $builder->get_object('BlastImporter_EValueCheck');
    my $widget = $builder->get_object('BlastImporter_EValueEntry');

    $widget->set_sensitive( $check->get_active );
}

sub on_BlastImporter_BitScoreCheck_toggled {
    my $self    = shift;
    my $builder = $self->uiBuilder;

    my $check  = $builder->get_object('BlastImporter_BitScoreCheck');
    my $widget = $builder->get_object('BlastImporter_BitScoreSpin');

    $widget->set_sensitive( $check->get_active );
}

1;
