package Gdraw2Gtk::EleTypeWizard;

use strict;
use feature qw/say switch/;
use Moose;
use File::ShareDir qw/:ALL/;
use Glib qw/TRUE FALSE/;
use Gtk2;

use constant {
    CLASS_BLOCK   => 'Gdraw2::Element::CairoBlock',
    CLASS_TAGSET  => 'Gdraw2::Element::CairoTagset',
    CLASS_MAPPING => 'Gdraw2::Element::CairoMapping',
    CLASS_RULER   => 'Gdraw2::Element::CairoRuler',
    CLASS_CURVE   => 'Gdraw2::Element::CairoDataCurve',
};

has uiBuilder => (
    is      => 'ro',
    isa     => 'Gtk2::Builder',
    default => sub { Gtk2::Builder->new }
);

sub BUILD {
    my $self = shift;

    my $builder = $self->uiBuilder;
    my $file_ui = dist_file( 'Gdraw2', 'EleTypeWizard.glade' );
    $builder->add_from_file($file_ui);

    my $wizard = $builder->get_object('EleTypeWizard');

    $wizard->signal_connect(
        'delete-event',
        sub {
            $wizard->response(1);
        }
    );
}

sub run {
    my $self    = shift;
    my $builder = $self->uiBuilder;

    my $wizard = $builder->get_object('EleTypeWizard');

    my $re = $wizard->run;
    $wizard->hide;
    
    if ( $re == 0 ) {
        if ($builder->get_object('EleType_BlockRadio')->get_active) {
            return CLASS_BLOCK;
        }
        if ($builder->get_object('EleType_TagsetRadio')->get_active) {
            return CLASS_TAGSET;
        }
        if ($builder->get_object('EleType_PairRadio')->get_active) {
            return undef;
        }
        if ($builder->get_object('EleType_MappingRadio')->get_active) {
            return CLASS_MAPPING;
        }
        if ($builder->get_object('EleType_DataCurveRadio')->get_active) {
            return CLASS_CURVE;
        }
        if ($builder->get_object('EleType_RulerRadio')->get_active) {
            return CLASS_RULER;
        }
    }
    else {
        return undef;
    }
}

1;
