package Gdraw2Gtk::GroupEditor;

use strict;
use feature qw/say switch/;
use Moose;
use File::ShareDir qw/:ALL/;
use Glib qw/TRUE FALSE/;
use Gtk2;

use constant {
    CLASS_BLOCK   => 'Gdraw2::Element::CairoBlock',
    CLASS_TAGSET  => 'Gdraw2::Element::CairoTagset',
    CLASS_MAPPING => 'Gdraw2::Element::CairoMapping',
    CLASS_RULER   => 'Gdraw2::Element::CairoRuler',
    CLASS_CURVE   => 'Gdraw2::Element::CairoDataCurve',
};

has uiBuilder => (
    is      => 'ro',
    isa     => 'Gtk2::Builder',
    default => sub { Gtk2::Builder->new }
);

sub BUILD {
    my $self = shift;

    my $builder = $self->uiBuilder;
    my $file_ui = dist_file( 'Gdraw2', 'GroupEditor.glade' );
    $builder->add_from_file($file_ui);
}

sub run {
    my ( $self, $params, $is_circ ) = @_;

    # create dialog
    my $builder = $self->uiBuilder;

    my $editor = $builder->get_object('GroupEditor');
    $editor->signal_connect(
        'delete-event',
        sub {
            $editor->response(1);
        }
    );

    # get widgets from builder
    my $name_entry        = $builder->get_object('GroupEditor_NameEntry');
    my $desc_entry        = $builder->get_object('GroupEditor_DescEntry');
    my $track_dist_spin   = $builder->get_object('GroupEditor_TrackDistSpin');
    my $start_spin        = $builder->get_object('GroupEditor_StartSpin');
    my $end_spin          = $builder->get_object('GroupEditor_EndSpin');
    my $left_margin_spin  = $builder->get_object('GroupEditor_LeftMarginSpin');
    my $right_margin_spin = $builder->get_object('GroupEditor_RightMarginSpin');
    my $circ_start_indent_spin =
      $builder->get_object('GroupEditor_CircStartIndentSpin');
    my $circ_end_indent_spin =
      $builder->get_object('GroupEditor_CircEndIndentSpin');

    # set properties to widget
    $name_entry->set_text( $params->{displayName} );
    $desc_entry->set_text( $params->{desc} );
    $track_dist_spin->set_value( $params->{trackDistance} );
    $start_spin->set_value( $params->{start} );
    $end_spin->set_value( $params->{end} );
    $left_margin_spin->set_value( $params->{leftMargin} );
    $right_margin_spin->set_value( $params->{rightMargin} );
    $circ_start_indent_spin->set_value( $params->{circStartIndent} );
    $circ_end_indent_spin->set_value( $params->{circEndIndent} );

    # apply topology
    if ($is_circ) {
        $left_margin_spin->set_sensitive(FALSE);
        $right_margin_spin->set_sensitive(FALSE);
    }
    else {
        $circ_start_indent_spin->set_sensitive(FALSE);
        $circ_end_indent_spin->set_sensitive(FALSE);
    }

    # run dialog
    my $re = $editor->run;
    $editor->hide;

    # accept changes
    if ( $re == 0 ) {
        $params->{displayName}     = $name_entry->get_text;
        $params->{desc}            = $desc_entry->get_text;
        $params->{trackDistance}   = $track_dist_spin->get_value;
        $params->{start}           = $start_spin->get_value;
        $params->{end}             = $end_spin->get_value;
        $params->{leftMargin}      = $left_margin_spin->get_value;
        $params->{rightMargin}     = $right_margin_spin->get_value;
        $params->{circStartIndent} = $circ_start_indent_spin->get_value;
        $params->{circEndIndent} = $circ_end_indent_spin->get_value;
    }

    return $re;
}

1;
