package Gdraw2Gtk::PageEditor;

use strict;
use feature qw/say switch/;
use Moose;
use File::ShareDir qw/:ALL/;
use Glib qw/TRUE FALSE/;
use Gtk2;

use constant {
    CLASS_BLOCK   => 'Gdraw2::Element::CairoBlock',
    CLASS_TAGSET  => 'Gdraw2::Element::CairoTagset',
    CLASS_MAPPING => 'Gdraw2::Element::CairoMapping',
    CLASS_RULER   => 'Gdraw2::Element::CairoRuler',
    CLASS_CURVE   => 'Gdraw2::Element::CairoDataCurve',
};

has uiBuilder => (
    is      => 'ro',
    isa     => 'Gtk2::Builder',
    default => sub { Gtk2::Builder->new }
);

sub BUILD {
    my $self = shift;

    my $builder = $self->uiBuilder;
    my $file_ui = dist_file( 'Gdraw2', 'PageEditor.glade' );
    $builder->add_from_file($file_ui);

    my $linear_rbut      = $builder->get_object('PageEditor_LinearRbut');
    my $circ_rbut        = $builder->get_object('PageEditor_CircularRbut');
    my $top_margin_spin  = $builder->get_object('PageEditor_TopMarginSpin');
    my $init_radius_spin = $builder->get_object('PageEditor_InitRadiusSpin');

    my $on_topo_change = sub {
        my $l_but = shift;
        if ( $l_but->get_active ) {
            $top_margin_spin->set_sensitive(TRUE);
            $init_radius_spin->set_sensitive(FALSE);
        }
        else {
            $top_margin_spin->set_sensitive(FALSE);
            $init_radius_spin->set_sensitive(TRUE);
        }
    };

    $linear_rbut->signal_connect( 'toggled', $on_topo_change );
}

sub run {
    my ( $self, $params ) = @_;

    my $builder = $self->uiBuilder;
    my $editor  = $builder->get_object('PageEditor');

    $editor->signal_connect(
        'delete-event',
        sub {
            $editor->response(1);
        }
    );

    # get widgets from builder
    my $width_spin       = $builder->get_object('PageEditor_WidthSpin');
    my $height_spin      = $builder->get_object('PageEditor_HeightSpin');
    my $group_dist_spin  = $builder->get_object('PageEditor_GroupDistSpin');
    my $top_margin_spin  = $builder->get_object('PageEditor_TopMarginSpin');
    my $init_radius_spin = $builder->get_object('PageEditor_InitRadiusSpin');
    my $title_font_entry = $builder->get_object('PageEditor_TitleFontEntry');
    my $title_size_spin  = $builder->get_object('PageEditor_TitleSizeSpin');
    my $text_font_entry  = $builder->get_object('PageEditor_TextFontEntry');
    my $text_size_spin   = $builder->get_object('PageEditor_TextSizeSpin');
    my $linear_rbut      = $builder->get_object('PageEditor_LinearRbut');
    my $circ_rbut        = $builder->get_object('PageEditor_CircularRbut');

    # set page attributes to dialog UI
    $width_spin->set_value( $params->{width} );
    $height_spin->set_value( $params->{height} );
    $group_dist_spin->set_value( $params->{groupDistance} );
    $top_margin_spin->set_value( $params->{topMargin} );
    $init_radius_spin->set_value( $params->{initRadius} );
    $title_font_entry->set_text( $params->{titleFont} );
    $title_size_spin->set_value( $params->{titleSize} );
    $text_font_entry->set_text( $params->{textFont} );
    $text_size_spin->set_value( $params->{textSize} );

    # set topology attribute
    $circ_rbut->set_active( $params->{isCircular} ? TRUE : FALSE );

    # run the page dialog
    my $re = $editor->run;

    # accept changes
    if ( $re == 0 ) {
        $params->{width}         = $width_spin->get_value;
        $params->{height}        = $height_spin->get_value;
        $params->{groupDistance} = $group_dist_spin->get_value;
        $params->{topMargin}     = $top_margin_spin->get_value;
        $params->{initRadius}    = $init_radius_spin->get_value;
        $params->{titleFont}     = $title_font_entry->get_text;
        $params->{titleSize}     = $title_size_spin->get_value;
        $params->{textFont}      = $text_font_entry->get_text;
        $params->{textSize}      = $text_size_spin->get_value;
        $params->{isCircular}    = $linear_rbut->get_active ? 0 : 1;
    }

    $editor->hide;
    return $re;
}

1;
