package Gdraw2Gtk::RichSeqImportConfig;

use strict;
use feature qw/say switch/;
use Moose;
use File::ShareDir qw/:ALL/;
use Glib qw/TRUE FALSE/;
use Gtk2;

use Gdraw2::Util qw/parse_color_str_16/;

use constant {
    COL_TRACK     => 0,
    COL_FEATURE   => 1,
    COL_ELE_TYPE  => 2,
    COL_TAG_NAME  => 3,
    COL_STROKE    => 4,
    COL_FILL      => 5,
    COL_COLOR_SET => 6
};

has uiBuilder => (
    is      => 'ro',
    isa     => 'Gtk2::Builder',
    default => sub { return Gtk2::Builder->new }
);

sub BUILD {
    my $self = shift;

    #
    # load GUI resources
    #
    my $builder = $self->uiBuilder;
    my $file_ui = dist_file( 'Gdraw2', 'FeatureImporter.glade' );
    $builder->add_from_file($file_ui);

    #
    # setup table
    #

    # create cell renderers
    my $col_track    = $builder->get_object('TrackList_TrackCol');
    my $col_feature  = $builder->get_object('TrackList_FeatureCol');
    my $col_ele_type = $builder->get_object('TrackList_EleTypeCol');
    my $col_tag_name = $builder->get_object('TrackList_TagNameCol');
    my $col_stroke   = $builder->get_object('TrackList_StrokeCol');
    my $col_fill     = $builder->get_object('TrackList_FillCol');

    my $r_name = Gtk2::CellRendererText->new;
    $col_track->pack_start( $r_name, TRUE );
    $col_track->add_attribute( $r_name, 'text', COL_TRACK );

    my $r_feature = Gtk2::CellRendererText->new;
    $col_feature->pack_start( $r_feature, TRUE );
    $col_feature->add_attribute( $r_feature, 'text', COL_FEATURE );

    my $r_ele_type = Gtk2::CellRendererText->new;
    $col_ele_type->pack_start( $r_ele_type, TRUE );
    $col_ele_type->add_attribute( $r_ele_type, 'text', COL_ELE_TYPE );

    my $r_tag_name = Gtk2::CellRendererText->new;
    $col_tag_name->pack_start( $r_tag_name, TRUE );
    $col_tag_name->add_attribute( $r_tag_name, 'text', COL_TAG_NAME );

    my $r_stroke = Gtk2::CellRendererText->new;
    $col_stroke->pack_start( $r_stroke, TRUE );
    $col_stroke->add_attribute( $r_stroke, 'cell-background', COL_STROKE );
    $col_stroke->add_attribute( $r_stroke, 'cell-background-set',
        COL_COLOR_SET );

    my $r_fill = Gtk2::CellRendererText->new;
    $col_fill->pack_start( $r_fill, TRUE );
    $col_fill->add_attribute( $r_fill, 'cell-background',     COL_FILL );
    $col_fill->add_attribute( $r_fill, 'cell-background-set', COL_COLOR_SET );

    # add some tracks
    my $model      = $builder->get_object('FeatureImporter_TrackModel');
    my $iter_track = $model->append(undef);
    $model->set( $iter_track, COL_TRACK, 'CDS track', COL_COLOR_SET,
        FALSE );
    my $iter_feat = $model->append($iter_track);
    $model->set(
        $iter_feat,      COL_FEATURE,  'CDS',       COL_ELE_TYPE,
        'block',         COL_TAG_NAME, 'locus_tag', COL_STROKE,
        '#000000000000', COL_FILL,     '#00007fff7fff'
    );

    # init UI state
    $self->on_FeatureImporter_RulerCheck_toggled;
    $self->on_FeatureImporter_RangeCheck_toggled;
    $self->on_FeatImporter_GCContentCheck_toggled;
    $self->on_FeatImporter_GCSkewCheck_toggled;
    
    #
    # connect signals
    #
    $builder->connect_signals( undef, $self );
    
    
}

sub drawGC {
    my $self = shift;
    my $builder = $self->uiBuilder;
    return $builder->get_object('FeatImporter_GCContentCheck')->get_active;
}

sub gcWindowSize {
    my $self = shift;
    my $builder = $self->uiBuilder;
    return $builder->get_object('FeatImporter_GCContentSizeSpin')->get_value;
}

sub gcWindowStep {
    my $self = shift;
    my $builder = $self->uiBuilder;
    return $builder->get_object('FeatImporter_GCContentStepSpin')->get_value;
}

sub drawSkew {
    my $self = shift;
    my $builder = $self->uiBuilder;
    return $builder->get_object('FeatImporter_GCSkewCheck')->get_active;
}

sub skewWindowSize {
    my $self = shift;
    my $builder = $self->uiBuilder;
    return $builder->get_object('FeatImporter_GCSkewSizeSpin')->get_value;
}

sub skewWindowStep {
    my $self = shift;
    my $builder = $self->uiBuilder;
    return $builder->get_object('FeatImporter_GCSkewStepSpin')->get_value;
}

sub drawRuler {
    my $self = shift;
    my $builder = $self->uiBuilder;
    return $builder->get_object('FeatureImporter_RulerCheck')->get_active;
}

sub rulerTickInterval {
    my $self = shift;
    my $builder = $self->uiBuilder;
    return $builder->get_object('FeatureImporter_RulerSpin')->get_value;
}

sub importPartial {
    my $self = shift;
    my $builder = $self->uiBuilder;
    return $builder->get_object('FeatureImporter_RangeCheck')->get_active;
}

sub partialStart {
    my $self = shift;
    my $builder = $self->uiBuilder;
    return $builder->get_object('FeatureImporter_StartSpin')->get_value;
}

sub partialEnd {
    my $self = shift;
    my $builder = $self->uiBuilder;
    return $builder->get_object('FeatureImporter_EndSpin')->get_value;
}

sub updateTableAt {
    my ( $self, $i ) = @_;

}

sub configTrackTemplate {
    my ( $self, $template ) = @_;

    # get objects
    my $builder     = $self->uiBuilder;
    my $editor      = $builder->get_object('ImportTrackEditor');
    my $rbut_block  = $builder->get_object('ImportTrackEditor_BlockTypeRadio');
    my $rbut_tagset = $builder->get_object('ImportTrackEditor_TagsetTypeRadio');
    my $entry_name  = $builder->get_object('ImportTrackEditor_TrackNameEntry');
    my $entry_desc  = $builder->get_object('ImportTrackEditor_TrackDescEntry');
    my $entry_name_tag = $builder->get_object('ImportTrackEditor_EleNameEntry');
    my $colorbut_stroke =
      $builder->get_object('ImportTrackEditor_StrokeColorBut');
    my $colorbut_fill = $builder->get_object('ImportTrackEditor_FillColorBut');
    my $list          = $builder->get_object('FeatImporter_TrackListModel');

    # set data to editor
    given ( $template->eleType ) {
        when ('block')  { $rbut_block->set_active(TRUE) }
        when ('tagset') { $rbut_tagset->set_active(TRUE) }
    }

    $entry_name->set_text( $template->name );
    $entry_desc->set_text( join ', ', @{ $template->features } );
    $entry_name_tag->set_text( join ', ', @{ $template->nameTags } );

    my $color_s = $template->strokeColor;
    my $color_f = $template->fillColor;
    $colorbut_stroke->set_color(
        Gtk2::Gdk::Color->new(
            65535 * $color_s->[0],
            65535 * $color_s->[1],
            65535 * $color_s->[2],
        )
    );
    $colorbut_fill->set_color(
        Gtk2::Gdk::Color->new(
            65535 * $color_f->[0],
            65535 * $color_f->[1],
            65535 * $color_f->[2],
        )
    );
    $colorbut_stroke->set_alpha( 65535 * $color_s->[3] );
    $colorbut_fill->set_alpha( 65535 * $color_f->[3] );

    # run editor
    my $re = $editor->run;
    $editor->hide;

    # apply changes
    if ( $re == 0 ) {
        $template->name( $entry_name->get_text );
        $template->features( [ split /,\s*/, $entry_desc->get_text ] );
        $template->nameTags( [ split /,\s*/, $entry_name_tag->get_text ] );

        if    ( $rbut_block->get_active )  { $template->eleType('block') }
        elsif ( $rbut_tagset->get_active ) { $template->eleType('tagset') }

        my $re_s = $colorbut_stroke->get_color;
        $color_s->[0] = $re_s->red / 65535;
        $color_s->[1] = $re_s->green / 65535;
        $color_s->[2] = $re_s->blue / 65535;
        $color_s->[3] = $colorbut_stroke->get_alpha / 65535;

        my $re_f = $colorbut_fill->get_color;
        $color_f->[0] = $re_f->red / 65535;
        $color_f->[1] = $re_f->green / 65535;
        $color_f->[2] = $re_f->blue / 65535;
        $color_f->[3] = $colorbut_fill->get_alpha / 65535;
    }

    return $re;
}

sub clearTrackTemplate {
    my $self    = shift;
    my $builder = $self->uiBuilder;
    my $list    = $builder->get_object('FeatImporter_TrackListModel');
    $list->clear;
    $self->{trackTemplates} = [];
}

sub run {
    my $self    = shift;
    my $builder = $self->uiBuilder;

    #
    # run dialog
    #
    my $ui = $builder->get_object('FeatImporter');
    my $re = $ui->run;
    $ui->hide;
    return $re;
}

sub getSelectedPath {
    my $self    = shift;
    my $builder = $self->uiBuilder;
    my $table   = $builder->get_object('FeatImporter_TrackList');
    my $sel     = $table->get_selection;
    return ( $sel->get_selected_rows )[-1];
}

sub setFeatTemplate {
    my ( $self, $iter ) = @_;

    my $builder = $self->uiBuilder;

    my $entry_tag_name = $builder->get_object('ImportTrackEditor_TagNameEntry');
    my $entry_feature  = $builder->get_object('ImportTrackEditor_FeatureEntry');
    my $rbut_type_block =
      $builder->get_object('ImportTrackEditor_BlockTypeRadio');
    my $rbut_type_tagset =
      $builder->get_object('ImportTrackEditor_TagsetTypeRadio');
    my $color_stroke =
      $builder->get_object('ImportTrackEditor_StrokeColorBut')->get_color;
    my $color_fill =
      $builder->get_object('ImportTrackEditor_FillColorBut')->get_color;

    my $str_stroke = sprintf( '#%04x%04x%04x',
        $color_stroke->red, $color_stroke->green, $color_stroke->blue );
    my $str_fill = sprintf( '#%04x%04x%04x',
        $color_fill->red, $color_fill->green, $color_fill->blue );
    my $ele_type;
    if ( $rbut_type_block->get_active ) {
        $ele_type = 'block';
    }
    elsif ( $rbut_type_tagset->get_active ) {
        $ele_type = 'tags';
    }

    $builder->get_object('FeatureImporter_TrackModel')->set(
        $iter,                     COL_FEATURE,
        $entry_feature->get_text,  COL_TAG_NAME,
        $entry_tag_name->get_text, COL_ELE_TYPE,
        $ele_type,                 COL_STROKE,
        $str_stroke,               COL_FILL,
        $str_fill,                 COL_COLOR_SET,
        TRUE
    );
}

sub loadFeatTemplate {
    my ( $self, $iter ) = @_;

    my $builder = $self->uiBuilder;
    my $model   = $builder->get_object('FeatureImporter_TrackModel');

    my $entry_tag_name = $builder->get_object('ImportTrackEditor_TagNameEntry');
    my $entry_feature  = $builder->get_object('ImportTrackEditor_FeatureEntry');
    my $rbut_type_block =
      $builder->get_object('ImportTrackEditor_BlockTypeRadio');
    my $rbut_type_tagset =
      $builder->get_object('ImportTrackEditor_TagsetTypeRadio');
    my $colorbut_stroke =
      $builder->get_object('ImportTrackEditor_StrokeColorBut');
    my $colorbut_fill = $builder->get_object('ImportTrackEditor_FillColorBut');

    $entry_feature->set_text( $model->get( $iter, COL_FEATURE ) );
    $entry_tag_name->set_text( $model->get( $iter, COL_TAG_NAME ) );

    my $ele_type = $model->get( $iter, COL_ELE_TYPE );
    if    ( $ele_type eq 'block' ) { $rbut_type_block->set_active(TRUE) }
    elsif ( $ele_type eq 'tags' )  { $rbut_type_tagset->set_active(TRUE) }
    else                           { confess "invalid element type: $ele_type" }

    my $color_stroke =
      Gtk2::Gdk::Color->parse( $model->get( $iter, COL_STROKE ) );
    my $color_fill = Gtk2::Gdk::Color->parse( $model->get( $iter, COL_FILL ) );
    $colorbut_stroke->set_color($color_stroke);
    $colorbut_fill->set_color($color_fill);
}

sub on_FeatImporter_AddTrackBut_clicked {
    my $self = shift;

    my $builder      = $self->uiBuilder;
    my $dialog_track = $builder->get_object('TrackEditor');
    my $name_entry   = $builder->get_object('TrackEditor_TrackNameEntry');
    my $model        = $builder->get_object('FeatureImporter_TrackModel');
    my $table        = $builder->get_object('FeatImporter_TrackList');

    my $re = $dialog_track->run;
    $dialog_track->hide;

    if ( $re == 0 ) {
        my $name     = $name_entry->get_text;
        my $path_sel = $self->getSelectedPath;
        my $iter;
        if ($path_sel) {
            my @i_sel    = $path_sel->get_indices;
            my $iter_rel = $model->get_iter_from_string( $i_sel[0] );
            $iter = $model->insert_after( undef, $iter_rel );
        }
        else {
            $iter = $model->append(undef);
        }

        $model->set( $iter, COL_TRACK, $name );
        my $path = $model->get_path($iter);
        $table->expand_to_path($path);
        $table->set_cursor($path);
    }

}

sub on_FeatureImporter_AddFeatureBut_clicked {
    my $self = shift;

    my $path_sel = $self->getSelectedPath;
    return if ( !defined $path_sel );

    my $builder     = $self->uiBuilder;
    my $dialog_feat = $builder->get_object('FeatureEditor');
    my $model       = $builder->get_object('FeatureImporter_TrackModel');
    my $table       = $builder->get_object('FeatImporter_TrackList');

    my $re = $dialog_feat->run;
    $dialog_feat->hide;

    if ( $re == 0 ) {
        my @i = $path_sel->get_indices;
        my $iter;
        if ( @i == 2 ) {
            my $iter_ref = $model->get_iter($path_sel);
            $iter = $model->insert_after( undef, $iter_ref );

        }
        elsif ( @i == 1 ) {
            my $iter_parent = $model->get_iter($path_sel);
            $iter = $model->append($iter_parent);
        }
        else {
            confess "invalid indices";
        }
        $self->setFeatTemplate($iter);

        my $path = $model->get_path($iter);
        $table->expand_to_path($path);
        $table->set_cursor($path);
    }
}

sub on_FeatImporter_EditBut_clicked {
    my $self = shift;

    my $path_sel = $self->getSelectedPath;
    return if ( !defined $path_sel );

    my $builder = $self->uiBuilder;
    my $model   = $builder->get_object('FeatureImporter_TrackModel');

    my $iter_sel = $model->get_iter($path_sel);
    my $depth    = $model->iter_depth($iter_sel);

    if ( $depth == 0 ) {
        my $dialog_track = $builder->get_object('TrackEditor');
        my $entry_name   = $builder->get_object('TrackEditor_TrackNameEntry');

        $entry_name->set_text( $model->get( $iter_sel, COL_TRACK ) );

        my $re = $dialog_track->run;
        $dialog_track->hide;

        if ( $re == 0 ) {
            $model->set( $iter_sel, COL_TRACK, $entry_name->get_text );
        }
    }
    elsif ( $depth == 1 ) {
        my $dialog_feat = $builder->get_object('FeatureEditor');

        $self->loadFeatTemplate($iter_sel);

        my $re = $dialog_feat->run;
        $dialog_feat->hide;

        if ( $re == 0 ) {
            $self->setFeatTemplate($iter_sel);
        }
    }
    else {
        confess "invalid depth $depth";
    }
}

sub on_FeatImporter_UpBut_clicked {
    my $self = shift;

    my $path_sel = $self->getSelectedPath;
    return if !defined $path_sel;

    my @i_ref = $path_sel->get_indices;
    return if $i_ref[-1] == 0;
    $i_ref[-1]--;

    my $builder = $self->uiBuilder;
    my $model   = $builder->get_object('FeatureImporter_TrackModel');
    my $table   = $builder->get_object('FeatImporter_TrackList');

    my $iter_sel = $model->get_iter($path_sel);
    my $iter_ref = $model->get_iter_from_string( join ':', @i_ref );

    $model->swap( $iter_sel, $iter_ref );
}

sub on_FeatImporter_DownBut_clicked {
    my $self = shift;

    my $path_sel = $self->getSelectedPath;
    return if !defined $path_sel;

    my $builder = $self->uiBuilder;
    my $model   = $builder->get_object('FeatureImporter_TrackModel');
    my $table   = $builder->get_object('FeatImporter_TrackList');

    my $iter_sel = $model->get_iter($path_sel);
    my $iter_ref = $model->iter_next($iter_sel);
    return if !defined $iter_ref;

    $model->swap( $iter_sel, $iter_ref );
}

sub on_FeatImporter_DelBut_clicked {
    my $self = shift;

    my $path_sel = $self->getSelectedPath;
    return if !defined $path_sel;

    # ask for sure
    my $builder = $self->uiBuilder;
    my $dialog  = $builder->get_object('DelObjMsg');
    my $re      = $dialog->run;
    $dialog->hide;
    return if $re eq 'yes';

    # do removal
    my $model = $builder->get_object('FeatureImporter_TrackModel');
    my $iter  = $model->get_iter($path_sel);
    $model->remove($iter);
}

sub on_FeatureImporter_RulerCheck_toggled {
    my $self = shift;
    
    my $builder = $self->uiBuilder;
    my $check = $builder->get_object('FeatureImporter_RulerCheck');
    
    my $state = $check->get_active;
    $builder->get_object('FeatureImporter_RulerSpin')->set_sensitive($state);
}

sub on_FeatureImporter_RangeCheck_toggled {
    my $self = shift;
    
    my $builder = $self->uiBuilder;
    my $check = $builder->get_object('FeatureImporter_RangeCheck');
    
    my $state = $check->get_active;
    
    $builder->get_object('FeatureImporter_StartSpin')->set_sensitive($state);
    $builder->get_object('FeatureImporter_EndSpin')->set_sensitive($state);
}

sub on_FeatImporter_GCContentCheck_toggled {
    my $self = shift;
    
    my $builder = $self->uiBuilder;
    my $check = $builder->get_object('FeatImporter_GCContentCheck');
    
    my $state = $check->get_active;
    
    $builder->get_object('FeatImporter_GCContentSizeSpin')->set_sensitive($state);
    $builder->get_object('FeatImporter_GCContentStepSpin')->set_sensitive($state);
}

sub on_FeatImporter_GCSkewCheck_toggled {
    my $self = shift;
    
    my $builder = $self->uiBuilder;
    my $check = $builder->get_object('FeatImporter_GCSkewCheck');
    
    my $state = $check->get_active;
    
    $builder->get_object('FeatImporter_GCSkewSizeSpin')->set_sensitive($state);
    $builder->get_object('FeatImporter_GCSkewStepSpin')->set_sensitive($state);
}

sub getTrackTemplates {
    my ( $self, $track_store, $feat_store ) = @_;

    my $i_track = 0;

    my $builder = $self->uiBuilder;
    my $model   = $builder->get_object('FeatureImporter_TrackModel');

    my $iter_track = $model->get_iter_first;

    while ( defined $iter_track ) {
        my $track_name = $model->get( $iter_track, COL_TRACK );
        push @$track_store, $track_name;

        my $iter_feat = $model->iter_children($iter_track);

        while ( defined $iter_feat ) {
            my ( $feat, $ele_type, $tags_str, $stroke_str, $fill_str ) =
              $model->get(
                $iter_feat, COL_FEATURE, COL_ELE_TYPE, COL_TAG_NAME,
                COL_STROKE, COL_FILL
              );

            my @tags = split /,\s*/, $tags_str;
            my @stroke = map { $_ / 65535 } parse_color_str_16($stroke_str);
            my @fill   = map { $_ / 65535 } parse_color_str_16($fill_str);

            $feat_store->{$feat} = [] if !exists $feat_store->{$feat};
            push @{ $feat_store->{$feat} },
              [ $i_track, $ele_type, [@tags], [ @stroke, 1 ], [ @fill, 1 ] ];
        }
        continue {
            $iter_feat = $model->iter_next($iter_feat);
        }
    }
    continue {
        $iter_track = $model->iter_next($iter_track);
        $i_track++;
    }

}

1;
