package Gdraw2Gtk::TrackEditor;

use strict;
use feature qw/say switch/;
use Moose;
use File::ShareDir qw/:ALL/;
use Glib qw/TRUE FALSE/;
use Gtk2;

use constant {
    CLASS_BLOCK   => 'Gdraw2::Element::CairoBlock',
    CLASS_TAGSET  => 'Gdraw2::Element::CairoTagset',
    CLASS_MAPPING => 'Gdraw2::Element::CairoMapping',
    CLASS_RULER   => 'Gdraw2::Element::CairoRuler',
    CLASS_CURVE   => 'Gdraw2::Element::CairoDataCurve',
};

has uiBuilder => (
    is      => 'ro',
    isa     => 'Gtk2::Builder',
    default => sub { Gtk2::Builder->new }
);

sub BUILD {
    my $self = shift;

    my $file_ui = dist_file( 'Gdraw2', 'TrackEditor.glade' );
    $self->uiBuilder->add_from_file($file_ui);
}

sub run {
    my ( $self, $params ) = @_;

    my $builder = $self->uiBuilder;

    my $editor = $builder->get_object('TrackEditor');
    $editor->signal_connect(
        'delete-event',
        sub {
            $editor->response(1);
        }
    );

    # get widgets
    my $name_entry       = $builder->get_object('TrackEditor_NameEntry');
    my $start_spin       = $builder->get_object('TrackEditor_StartSpin');
    my $end_spin         = $builder->get_object('TrackEditor_EndSpin');
    my $ele_width_spin   = $builder->get_object('TrackEditor_EleWidthSpin');
    my $guide_line_check = $builder->get_object('TrackEditor_GuideLineCheck');

    # set track properties to UI
    $name_entry->set_text( $params->{displayName} );
    $start_spin->set_value( $params->{start} );
    $end_spin->set_value( $params->{end} );
    $ele_width_spin->set_value( $params->{elementWidth} );
    if ( $params->{showGuideLine} ) {
        $guide_line_check->set_active(TRUE);
    }
    else {
        $guide_line_check->set_active(FALSE);
    }

    # run dialog
    my $re = $editor->run;

    # apply if accepted
    if ( $re == 0 ) {
        $params->{displayName}  = $name_entry->get_text;
        $params->{start}        = $start_spin->get_value;
        $params->{end}          = $end_spin->get_value;
        $params->{elementWidth} = $ele_width_spin->get_value;
        $params->{showGuideLine} = $guide_line_check->get_active ? 1 : 0;
    }

    $editor->hide;

    return $re;
}

1;
