#!/usr/bin/perl

use common::sense;
use Cairo;

use Gdraw2::CairoTrackPage;
use Gdraw2::CairoTrackGroup;
use Gdraw2::CairoTrack;
use Gdraw2::Element::CairoTagset;
use Gdraw2::Element::CairoBlock;
use Gdraw2::Element::CairoRuler;
use Gdraw2::Util::RGBAColor;

my $surface = Cairo::SvgSurface->create( 'circ.svg', 600, 600 );
my $cr = Cairo::Context->create($surface);

my $page = Gdraw2::CairoTrackPage->new(
	cairoContext => $cr,
	width        => 600,
	height       => 600,
	textFont     => 'Arial',
	textSize     => 8,
	isCircular   => 1
);

my $groupA = Gdraw2::CairoTrackGroup->new(
	start => 1,
	end   => 1000
);
$page->addChild($groupA);

my $track_ruler = Gdraw2::CairoTrack->new(
	start         => 1,
	end           => 750,
	showGuideLine => 0
);
$groupA->addChild($track_ruler);

my $ruler = Gdraw2::Element::CairoRuler->new(
	position   => [ 1, 750 ],
	gridLength => 100
);
$track_ruler->addChild($ruler);

my $trackA1 = Gdraw2::CairoTrack->new(
	start => 1,
	end   => 750
);
$groupA->addChild($trackA1);

for my $pos ( 1, 50, 100, 250, 500, 600, 700, 750 ) {
	my $tag = Gdraw2::Element::CairoTagset->new(
		position    => [$pos,$pos+5,$pos+10],
		displayName => "tag_$pos",
	);
	$trackA1->addChild($tag);
}

for my $pos ( 1 .. 5 ) {
	my $start = $pos * 100 + 1;
	my $end   = $start + 150 - 1;
	my $block = Gdraw2::Element::CairoBlock->new(
		elementAnchor => 'center',
		strand        => $pos % 2 == 0 ? 1 : -1,
		fillColor     => Gdraw2::Util::RGBAColor->new( 0, 1, 1, 0.5 ),
		position      => [ $start, $end - 70, $end - 60, $end ],
		displayName   => "block $start-$end"
	);
	$trackA1->addChild($block);
}

for my $pos ( 1 .. 5 ) {
	my $start = $pos * 100 + 1;
	my $end   = $start + 3 - 1;
	my $block = Gdraw2::Element::CairoBlock->new(
		elementAnchor => 'center',
		strand        => $pos % 2 == 0 ? 1 : -1,
		fillColor     => Gdraw2::Util::RGBAColor->new( 1, 0, 1, 0.5 ),
		position    => [ $start, $end ],
		displayName => "short $start-$end"
	);
	$trackA1->addChild($block);
}

$trackA1->addChild(
	Gdraw2::Element::CairoBlock->new(
		position => [ -50, 50 ],
		strand   => 1,
		fillColor => Gdraw2::Util::RGBAColor->new( 1, 0, 0, 0.5 ),
	)
);
$trackA1->addChild(
	Gdraw2::Element::CairoBlock->new(
		position => [ 700, 800 ],
		strand   => -1,
		fillColor => Gdraw2::Util::RGBAColor->new( 1, 0, 0, 0.5 ),
	)
);

$page->draw;
