#!/usr/bin/perl

use strict;

use Glib qw/TRUE FALSE/;
use Cairo;

use Gdraw2::Element::CairoBlock;
use Gdraw2::Element::CairoMapping;
use Gdraw2::Element::CairoDataCurve;
use Gdraw2::CairoTrackPage;
use Gdraw2::CairoTrackGroup;
use Gdraw2::CairoTrack;
use Gdraw2::Util::RGBColor;
use Gdraw2::Util::RGBAColor;

use YAML::Syck qw/Dump Load DumpFile LoadFile/;

my $surface = Cairo::PdfSurface->create( 'full_cairo.pdf', 1600, 600 );
my $cr = Cairo::Context->create($surface);

my $page = Gdraw2::CairoTrackPage->new(
	cairoContext => $cr,
	width        => 1600,
	height       => 600
);

my $groupA = Gdraw2::CairoTrackGroup->new(
	start       => 1,
	end         => 1000,
	leftMargin  => 50,
	rightMargin => 50
);
$page->addChild($groupA);
my $groupB = Gdraw2::CairoTrackGroup->new(
	end         => 600,
	leftMargin  => 250,
	rightMargin => 150
);
$page->addChild($groupB);

my $trackA1 = Gdraw2::CairoTrack->new(
	start => 1,
	end   => 1000
);

my $trackA2 = Gdraw2::CairoTrack->new(
	start => 501,
	end   => 1000
);
$groupA->addChild($trackA1);
$groupA->addChild($trackA2);

my $trackB = Gdraw2::CairoTrack->new(
	start => 1,
	end   => 600
);
$groupB->addChild($trackB);

my $eleA1_1 = Gdraw2::Element::CairoBlock->new(
	displayName   => 'element A1-1',
	nameAnchor    => 'right',
	elementAnchor => 'right',
	strand        => -1,
	position      => [ 1, 200 ]
);
my $eleA1_2 = Gdraw2::Element::CairoBlock->new(
	displayName   => 'element A1-2',
	nameAnchor    => 'center',
	elementAnchor => 'center',
	position      => [ 50, 250 ]
);
my $eleA1_3 = Gdraw2::Element::CairoBlock->new(
	displayName => 'element A1-3',
	strand      => 1,
	position    => [ 300, 400, 450, 600 ]
);
my $eleA1_4 = Gdraw2::Element::CairoBlock->new(
	displayName => 'element A1-4',
	strand      => -1,
	position    => [ 600, 650, 800, 1000 ]
);
my $mapA1 = Gdraw2::Element::CairoMapping->new(
	displayName => 'aaaaa',
	showName    => 0,
	layer       => 'top',
	fillColor   => Gdraw2::Util::RGBAColor->new( 0.5, 0.5, 1, 0.5 ),
	position      => [ 1, 200 ],
	extGroupIndex => 1,
	extStart      => 1,
	extEnd        => 200
);
$trackA1->addChild($eleA1_1);
$trackA1->addChild($eleA1_2);
$trackA1->addChild($eleA1_3);
$trackA1->addChild($eleA1_4);
$trackA1->addChild($mapA1);

my $eleA2_1 = Gdraw2::Element::CairoBlock->new(
	displayName => 'element A2_1',
	strand      => -1,
	position    => [ 600, 650, 680, 950 ]
);
my $eleA2_2 = Gdraw2::Element::CairoBlock->new(
	displayName => 'element A2_2',
	strand      => 1,
	position    => [ 600, 630 ]
);

$trackA2->addChild($eleA2_1);
$trackA2->addChild($eleA2_2);

my $eleB_1 = Gdraw2::Element::CairoBlock->new(
	displayName => 'element B-1',
	strand      => -1,
	position    => [ 1, 150, 201, 750 ]
);
my $eleB_2 = Gdraw2::Element::CairoBlock->new(
	displayName => 'element B-2',
	strand      => 1,
	position    => [ -50, 150 ]
);
my $mapB = Gdraw2::Element::CairoMapping->new(
	displayName => 'bbbbb',
	layer       => 'top',
	fillColor   => Gdraw2::Util::RGBAColor->new( 1, 0.5, 0.5, 0.5 ),
	position => [ 401, 600 ],
	extStart => 701,
	extEnd   => 900
);
my $curve = Gdraw2::Element::CairoDataCurve->new(
	displayName => 'curve',
	layer => 'top',
	position => [401,600],
	middleLine => 3,
	data => [1,2,3,4,5]
);

$trackB->addChild($eleB_1);
$trackB->addChild($eleB_2);
$trackB->addChild($mapB);
$trackB->addChild($curve);

$page->draw;

my $hash = $page->toHash;

DumpFile( 'full_cairo.yaml', $hash );

