#!/usr/bin/perl

use common::sense;

use Cairo;

use Gdraw2::Element::CairoBlock;
use Gdraw2::Element::CairoMapping;
use Gdraw2::Element::CairoRuler;
use Gdraw2::Element::CairoTagset;
use Gdraw2::CairoTrackPage;
use Gdraw2::CairoTrackGroup;
use Gdraw2::CairoTrack;
use Gdraw2::Util::RGBColor;
use Gdraw2::Util::RGBAColor;

my $surface = Cairo::PdfSurface->create( 'short.pdf', 600, 600 );
my $cr = Cairo::Context->create($surface);

my $page = Gdraw2::CairoTrackPage->new(
	cairoContext => $cr,
	width        => 600,
	height       => 600
);

my $group1 = Gdraw2::CairoTrackGroup->new(
	start       => -10,
	end         => 10,
	leftMargin  => 50,
	rightMargin => 50
);
$page->addChild($group1);

my $track_ruler = Gdraw2::CairoTrack->new(
	start         => -10,
	end           => 10,
	showGuideLine => 0
);
$group1->addChild($track_ruler);
my $ruler = Gdraw2::Element::CairoRuler->new(
	position   => [ -10, 10 ],
	gridLength => 1
);
$track_ruler->addChild($ruler);

my $track1 = Gdraw2::CairoTrack->new(
	start => -10,
	end   => 10,
);
$group1->addChild($track1);

my @loc_tag;
for my $i ( -10 .. 10 ) {
	next if $i == 0;
	push @loc_tag,$i;
	$track1->addChild(
		Gdraw2::Element::CairoBlock->new(
			position => [ $i,$i ],
			fillColor   => Gdraw2::Util::RGBAColor->new( 0, 1, 1, 0.5 ),
			elementAnchor => 'center',
			nameAnchor => 'center',
			strand      => 1,
			displayName => "at$i"
		)
	);
}

$track1->addChild(Gdraw2::Element::CairoTagset->new(
    position => [@loc_tag]
));

$page->draw;
